;;; neocities.el --- Implementation of neocities API -*- lexical-binding: t -*-

;; Version: 0.2.3
;; Homepage: https://shrine.stiernstrom.se/packages/neocities.el
;; Package-requires: ((emacs "28.1"))

;;; Commentary:

;; Relies soley on url package except for uploads where we need a
;; little help by curl as binary uploads using url seem neigh
;; impossible.

(require 'url)
(require 'cl-lib)
(require 'project)

;;; Code:

(defcustom neocities-keys nil
 "API keys retrieved through /api/key keyed by project name"
 :type '(alist :key-type string :value-type string)
 :group 'neocities)

(defun neocities-current-project-key ()
 "Return api key for current project name."
 (cdr (assoc (project-name (project-current)) neocities-keys)))

(defun neocities-key (username password)
 "Return or create api key for use authenticated by USERNAME and PASSWORD."
 (with-current-buffer
  (url-retrieve-synchronously
   (format "https://%s:%s@neocities.org/api/key" username password))
  (json-parse-string
   (buffer-substring url-http-end-of-headers (point-max))
   :object-type 'plist)))

(defun neocities-info (&optional sitename)
 "Get information about SITENAME."
 (let ((url-request-extra-headers
	(unless sitename (list (cons "Authorization" (format "Bearer %s" (neocities-current-project-key)))))))
  (with-current-buffer
   (url-retrieve-synchronously
    (concat "https://neocities.org/api/info"
     (when sitename (format "?sitename=%s" sitename))))
   (json-parse-string
    (buffer-substring url-http-end-of-headers (point-max))
    :object-type 'plist))))

(defun neocities-list (&optional path)
 "List files in optional PATH."
 (let ((url-request-extra-headers
	(list (cons "Authorization" (format "Bearer %s" (neocities-current-project-key))))))
  (with-current-buffer
   (url-retrieve-synchronously
    (concat "https://neocities.org/api/list"
     (when path (format "?path=%s" path))))
   (json-parse-string
    (buffer-substring url-http-end-of-headers (point-max))
    :object-type 'plist))))

(defun neocities-delete (&rest files)
 "Delete FILES from server."
 (let ((url-request-extra-headers
	(list
	 (cons "Authorization" (format "Bearer %s" (neocities-current-project-key)))
	 (cons "Content-Type" "application/x-www-form-urlencoded")))
       (url-request-method "POST")
       (url-request-data
	(mapconcat
	 (lambda (f)
	  (concat
	   (url-hexify-string "filenames[]")
	   "="
	   (url-hexify-string f)))
	 files "&")))
  (with-current-buffer
   (url-retrieve-synchronously "https://neocities.org/api/delete")
   (json-parse-string
    (buffer-substring url-http-end-of-headers (point-max))
    :object-type 'plist))))

(defun neocities-upload (&rest files)
 "Upload FILES using curl command.

  Any attemp to use url package ran into us not being able to
  send binary files and so we use curl directly instead."
 (let* ((shell-command
	 (concat
	  (format "curl -s -H \"Authorization: Bearer %s\" " (neocities-current-project-key))
	  (mapconcat (lambda (file)
		      (format "-F \"%s=@%s\""
		       (file-relative-name file
			(project-root (project-current)))
		       file)) (flatten-list files) " ")
	  " https://neocities.org/api/upload"))
	(curl-output
	 (shell-command-to-string shell-command)))
  (condition-case nil
   (json-parse-string curl-output :object-type 'plist)
   (error (princ shell-command)))))

(provide 'neocities)

;;; neocities.el ends here
